-- Auctioneer
-- Written by Norganna
--
-- This is an addon for World of Warcraft that works in combination with 
-- LootLink to add statistical history to the auction data that is collected
-- when the auction is scanned, so that you can easily determine what price
-- you will be able to sell an item for at auction or at a vendor whenever you
-- mouse-over an item in the game
--
-- (LootLink is an in-game item database)
--

AUCTIONEER_VERSION 				= "1.0";
AUCTIONEER_WEBSITE				= "http://norgs.internutter.org/auctioneer.htm";

AUCTIONEER_LOADED				= "Auctioneer v%s loaded";
AUCTIONEER_NO_HOOKS				= "Auctioneer could not find the methods to hook.\nRecommendation: use the Auctioneer-provided LootLink.lua from :\n"..AUCTIONEER_WEBSITE;
AUCTIONEER_NO_AUCTION_HOOKS		= "Auctioneer incapable of gathering data.";
AUCTIONEER_NO_TOOLTIP_HOOKS		= "Auctioneer incapable of presenting data.";

AUCTIONEER_NEVER_SEEN			= "Never seen at auction";

AUCTIONEER_SEEN_TIMES			= "%d times at auction";
AUCTIONEER_AVERAGE_MORE_ONE		= "Avg: x%d, %s, %s, %s";
AUCTIONEER_AVERAGE_ONE			= "Avg: %s, %s, %s";
AUCTIONEER_SUGGESTED_MORE_ONE	= "Sug: (for %d) %s, %s, %s";
AUCTIONEER_PERCENT_BID_BUYOUT	= "Pct: bidded=%d%%, buyout=%d%%";
AUCTIONEER_BUY_SELL_MORE_ONE	= "[x%d] Buy%s %s, Sell%s %s";
AUCTIONEER_BUY_SELL_ONE			= "Buy%s %s, Sell%s %s";

AUCTIONEER_NONE					= "none";

GSC_GOLD						= "|cffe4cf65%dg|r";
GSC_SILVER						= "|cffbcb9bc%ds|r";
GSC_COPPER						= "|cffc38f6f%dc|r";
GSC_NONE						= "|cffa0a0a0none|r";
GSC_NONE						= "|cffa0a0a0%s|r";


Auction_DoneItems = {};
local function Auctioneer_AuctionStart_Hook()
	Auctioneer_Old_AuctionStart_Hook();
	Auction_DoneItems = {};
end

local function Auctioneer_AuctionEntry_Hook(page, index, name)
	Auctioneer_Old_AuctionEntry_Hook(page,index,name);
	if (not page) then
		return;
	end
	if (not Auction_DoneItems[page]) then
		Auction_DoneItems[page] = {};
	end
	if (Auction_DoneItems[page][index]) then
		return;
	end
	Auction_DoneItems[page][index] = true;

	local aiName, aiTexture, aiCount, aiQuality, aiCanUse, aiLevel, aiMinBid, aiMinIncrement, aiBuyoutPrice, aiBidAmount, aiHighBidder, aiOwner = GetAuctionItemInfo("list", index);

	if (aiCount < 1) then aiCount = 1; end

	local ahCount = 0;
	local ahMinCount = 0;
	local ahMinPrice = 0;
	local ahBidCount = 0;
	local ahBidPrice = 0;
	local ahBuyCount = 0;
	local ahBuyPrice = 0;
	if ((ItemLinks[name].ahCount) and  (ItemLinks[name].ahCount > 0)) then
		ahCount    = ItemLinks[name].ahCount;
	end
	if ((ItemLinks[name].ahMinCount) and (ItemLinks[name].ahMinCount > 0)) then
		ahMinCount = ItemLinks[name].ahMinCount;
	end
	if ((ItemLinks[name].ahMinPrice) and (ItemLinks[name].ahMinPrice > 0)) then
		ahMinPrice = ItemLinks[name].ahMinPrice;
	end
	if ((ItemLinks[name].ahBidCount) and (ItemLinks[name].ahBidCount > 0)) then
		ahBidCount = ItemLinks[name].ahBidCount;
	end
	if ((ItemLinks[name].ahBidPrice) and (ItemLinks[name].ahBidPrice > 0)) then
		ahBidPrice = ItemLinks[name].ahBidPrice;
	end
	if ((ItemLinks[name].ahBuyCount) and (ItemLinks[name].ahBuyCount > 0)) then
		ahBuyCount = ItemLinks[name].ahBuyCount;
	end
	if ((ItemLinks[name].ahBuyPrice) and (ItemLinks[name].ahBuyPrice > 0)) then
		ahBuyPrice = ItemLinks[name].ahBuyPrice;
	end

	ItemLinks[name].ahCount = ahCount + 1;
	ItemLinks[name].ahMinCount = ahMinCount + aiCount;
	ItemLinks[name].ahMinPrice = ahMinPrice + aiMinBid;
	if (aiBuyoutPrice > 0) then
		ItemLinks[name].ahBuyCount = ahBuyCount + aiCount;
		ItemLinks[name].ahBuyPrice = ahBuyPrice + aiBuyoutPrice;
	end
	if (aiBidAmount > 0) then 
		ItemLinks[name].ahBidCount = ahBidCount + aiCount;
		ItemLinks[name].ahBidPrice = ahBidPrice + aiBidAmount;
	end
end

local function Auctioneer_GetGSC(money)
	local g = math.floor(money / 10000);
	local s = math.floor((money - (g*10000)) / 100);
	local c = math.floor(money - (g*10000) - (s*100));
	return g,s,c;
end

local function Auctioneer_GetTextGSC(money)
	local g, s, c = Auctioneer_GetGSC(money);
	local gsc = "";
	if (g > 0) then
		gsc = format(GSC_GOLD, g);
		if ((s > 0) or (c > 0)) then
			gsc = gsc.." "..format(GSC_SILVER, s);
			if (c > 0) then
				gsc = gsc.." "..format(GSC_COPPER, c);
			end
		end
	elseif (s > 0) then
		gsc = format(GSC_SILVER, s);
		if (c > 0) then
			gsc = gsc.." "..format(GSC_COPPER, c);
		end
	elseif (c > 0) then
		gsc = gsc..format(GSC_COPPER, c);
	else
		gsc = gsc..format(GSC_NONE, AUCTIONEER_NONE);
	end

	return gsc;
end

local function Auctioneer_Tooltip_Hook(frame, name, money, count, data)
	if (data) then
		local ahCount = 0;
		local ahMinCount = 0;
		local ahMinPrice = 0;
		local ahBidCount = 0;
		local ahBidPrice = 0;
		local ahBuyCount = 0;
		local ahBuyPrice = 0;
		if ((data.ahCount) and (data.ahCount > 0)) then ahCount = data.ahCount; end
		if ((data.ahMinCount) and (data.ahMinCount > 0)) then ahMinCount = data.ahMinCount; end
		if ((data.ahMinPrice) and (data.ahMinPrice > 0)) then ahMinPrice = data.ahMinPrice; end
		if ((data.ahBidCount) and (data.ahBidCount > 0)) then ahBidCount = data.ahBidCount; end
		if ((data.ahBidPrice) and (data.ahBidPrice > 0)) then ahBidPrice = data.ahBidPrice; end
		if ((data.ahBuyCount) and (data.ahBuyCount > 0)) then ahBuyCount = data.ahBuyCount; end
		if ((data.ahBuyPrice) and (data.ahBuyPrice > 0)) then ahBuyPrice = data.ahBuyPrice; end
			
		if (ahCount == 0) then
			frame:AddLine(AUCTIONEER_NEVER_SEEN, 0.5, 0.8, 0.5);
		else
			local avgQty = math.floor(ahMinCount / ahCount);
			local avgMin = math.floor(ahMinPrice / ahMinCount);

			local bidPct = math.floor(ahBidCount / ahMinCount * 100);
			local avgBid = 0;
			if (ahBuyCount > 0) then
				avgBid = math.floor(ahBidPrice / ahBidCount);
			end
			
			local buyPct = math.floor(ahBuyCount / ahMinCount * 100);
			local avgBuy = 0;
			if (ahBuyCount > 0) then
				avgBuy = math.floor(ahBuyPrice / ahBuyCount);
			end

			frame:AddLine(format(AUCTIONEER_SEEN_TIMES, ahCount), 0.5,0.8,0.1);
			if (avgQty > 1) then
				frame:AddLine(format(AUCTIONEER_AVERAGE_MORE_ONE, avgQty, Auctioneer_GetTextGSC(avgMin), Auctioneer_GetTextGSC(avgBid), Auctioneer_GetTextGSC(avgBuy)), 0.1,0.8,0.5);
			else
				frame:AddLine(format(AUCTIONEER_AVERAGE_ONE, Auctioneer_GetTextGSC(avgMin), Auctioneer_GetTextGSC(avgBid), Auctioneer_GetTextGSC(avgBuy)), 0.1,0.8,0.5);
			end
			if (count > 1) then
				frame:AddLine(format(AUCTIONEER_SUGGESTED_MORE_ONE, count, Auctioneer_GetTextGSC(avgMin*count), Auctioneer_GetTextGSC(avgBid*count), Auctioneer_GetTextGSC(avgBuy*count)), 0.5,0.5,0.8);
			end
			frame:AddLine(format(AUCTIONEER_PERCENT_BID_BUYOUT, bidPct, buyPct), 0.1,0.5,0.8);
		end
	end


	--print2("Data.item = " .. data.item);
	if ( data.item ) then
		local i,j, baseItemID = string.find(data.item, "^(%d+):");
		local itemInfo = nil;
		if (baseItemID) then
			--print2("Base ItemID = "..baseItemID);
			itemInfo = Auctioneer_BasePrices[baseItemID+0];
		end
		local sellNote = "";
		local buyNote = "";
		local sell = money;
		local buy = sell * 4;
		if (itemInfo) then
			--print2("Base Item: "..itemInfo.name..", buy "..itemInfo.buy..", sell "..itemInfo.sell);
			buyNote = "*"
			buy = itemInfo.buy;
			if ((not sell) or (sell == "") or (sell <= 0))  then
				sellNote = "*"
				sell = itemInfo.sell;
			end;
		end
			
		if ((buy > 0) or (sell > 0)) then
			if (count > 1) then
				local bgsc = Auctioneer_GetTextGSC(buy*count);
				local sgsc = Auctioneer_GetTextGSC(sell*count);
				frame:AddLine(format(AUCTIONEER_BUY_SELL_MORE_ONE, count, buyNote, bgsc, sellNote, sgsc), 0.8, 0.5, 0.1);
			else
				local bgsc = Auctioneer_GetTextGSC(buy);
				local sgsc = Auctioneer_GetTextGSC(sell);
				frame:AddLine(format(AUCTIONEER_BUY_SELL_ONE, buyNote, bgsc, sellNote, sgsc), 0.8, 0.5, 0.1);
			end
		end
	end

	Auctioneer_Old_Tooltip_Hook(frame, name, money, count, data);
end

Auctioneer_HookedAuctionFunctions = {
	"LootLink_AuctionStart_Hook",
	"LootLink_AuctionEntry_Hook"
};

Auctioneer_HookedTooltipFunctions = {
	"LootLink_Tooltip_Hook",
};

function Auctioneer_DoesFunctionsExist(funcs)
	for k, v in funcs do
		if ( not getglobal(v) ) then
			return false;
		end
	end
	return true;
end

function Auctioneer_Print(msg)
	if ( DEFAULT_CHAT_FRAME ) then 
		DEFAULT_CHAT_FRAME:AddMessage(msg, 0.8, 0.8, 0.2);
	end
end

function Auctioneer_OnLoad()

	local hasAuctionFunctions = Auctioneer_DoesFunctionsExist(Auctioneer_HookedAuctionFunctions);
	local hasTooltipFunctions = Auctioneer_DoesFunctionsExist(Auctioneer_HookedTooltipFunctions);
	
	if ( not hasAuctionFunctions ) or ( not hasTooltipFunctions ) then
		Auctioneer_Print(AUCTIONEER_NO_HOOKS);
		if ( not hasAuctionFunctions ) then
			Auctioneer_Print(AUCTIONEER_NO_AUCTION_HOOKS);
		end
		if ( not hasTooltipFunctions ) then
			Auctioneer_Print(AUCTIONEER_NO_TOOLTIP_HOOKS);
		end
	end
	Auctioneer_Old_Tooltip_Hook = LootLink_Tooltip_Hook;
	LootLink_Tooltip_Hook = Auctioneer_Tooltip_Hook;

	Auctioneer_Old_AuctionStart_Hook = LootLink_AuctionStart_Hook;
	LootLink_AuctionStart_Hook = Auctioneer_AuctionStart_Hook;

	Auctioneer_Old_AuctionEntry_Hook = LootLink_AuctionEntry_Hook;
	LootLink_AuctionEntry_Hook = Auctioneer_AuctionEntry_Hook;

	Auctioneer_Print(format(AUCTIONEER_LOADED, AUCTIONEER_VERSION));
end


