-- Enchantrix
-- Written by Norganna
--
-- This is an addon for World of Warcraft that works in combination with 
-- LootLink to add an item breaks down into list to the items that you 
-- mouse-over in the game
--
-- (LootLink is an in-game item database)
--

EnchantedLocal = {};

ElementNames = {};
ElementNames["DustDream"] = "Dream Dust";
ElementNames["DustIllusion"] = "Illusion Dust";
ElementNames["DustSoul"] = "Soul Dust";
ElementNames["DustStrange"] = "Strange Dust";
ElementNames["DustVision"] = "Vision Dust";
ElementNames["EssenceAstralLarge"] = "Greater Astral Essence";
ElementNames["EssenceAstralSmall"] = "Lesser Astral Essence";
ElementNames["EssenceEternalLarge"] = "Greater Eternal Essence";
ElementNames["EssenceEternalSmall"] = "Lesser Eternal Essence";
ElementNames["EssenceMagicLarge"] = "Greater Magic Essence";
ElementNames["EssenceMagicSmall"] = "Lesser Magic Essence";
ElementNames["EssenceMysticLarge"] = "Greater Mystic Essence";
ElementNames["EssenceMysticSmall"] = "Lesser Mystic Essence";
ElementNames["EssenceNetherLarge"] = "Greater Nether Essence";
ElementNames["EssenceNetherSmall"] = "Lesser Nether Essence";
ElementNames["ShardBrilliantLarge"] = "Large Brilliant Shard";
ElementNames["ShardBrilliantSmall"] = "Small Brilliant Shard";
ElementNames["ShardGlimmeringLarge"] = "Large Glimmering Shard";
ElementNames["ShardGlimmeringSmall"] = "Small Glimmering Shard";
ElementNames["ShardGlowingLarge"] = "Large Glowing Shard";
ElementNames["ShardGlowingSmall"] = "Small Glowing Shard";
ElementNames["ShardRadientLarge"] = "Large Radient Shard";
ElementNames["ShardRadientSmall"] = "Small Radient Shard";

local function Enchantrix_Tooltip_Hook(frame, name, money, count, data)
	if ((EnchantedItems[name]) or (EnchantedLocal[name])) then
		local total = 0;
		local disenchantsTo = {};
		if (EnchantedItems[name]) then
			for name, count in EnchantedItems[name] do
				disenchantsTo[name] = {};
				disenchantsTo[name].inBuilt = count;
				disenchantsTo[name].myCount = 0;
				total = total + count;
			end
		end
		if (EnchantedLocal[name]) then
			for name, count in EnchantedLocal[name] do
				if (not disenchantsTo[name]) then
					disenchantsTo[name] = {};
					disenchantsTo[name].inBuilt = 0;
				end
				disenchantsTo[name].myCount = count;
				total = total + count;
			end
		elseif (total < 10) then
			total = 0
			disenchantsTo = {};
		end

		if (total > 0) then
			frame:AddLine("Disenchants into:", 0.7,0.7,0.2);
			for name, counts in disenchantsTo do
				local count = counts.inBuilt + counts.myCount;
				local pct = math.floor(count / total * 100);
				frame:AddLine(" " .. name .. ": " .. pct .. "%; " .. count .. " (" .. counts.myCount .. " by me)", 0.7,0.7,0.2);
			end
		end
	end

	Enchantrix_Old_Tooltip_Hook(frame, name, money, count, data);
end

function Enchantrix_OnLoad()
	Enchantrix_Old_Tooltip_Hook = LootLink_Tooltip_Hook;
	LootLink_Tooltip_Hook = Enchantrix_Tooltip_Hook;

	this:RegisterEvent("SPELLCAST_FAILED");
	this:RegisterEvent("SPELLCAST_INTERRUPTED");
	this:RegisterEvent("SPELLCAST_START");
	this:RegisterEvent("SPELLCAST_STOP");
	this:RegisterEvent("ITEM_PUSH");
	this:RegisterEvent("UNIT_INVENTORY_CHANGED");
	
	RegisterForSave("EnchantedLocal");

	Enchantrix_DisenchantCount = 0;
	Enchantrix_DisenchantResult = {};
	Enchantrix_Disenchanting = false;
	Enchantrix_WaitingPush = false;

	if ( DEFAULT_CHAT_FRAME ) then 
		DEFAULT_CHAT_FRAME:AddMessage("Enchantrix v1.0 loaded", 0.8, 0.8, 0.2);
	end
end

function Enchantrix_NameFromLink(link)
	local name;
	if( not link ) then
		return nil;
	end
	for name in string.gfind(link, "|c%x+|Hitem:%d+:%d+:%d+:%d+|h%[(.-)%]|h|r") do
		return name;
	end
	return nil;
end

function Enchantrix_TakeInventory()
	local bagid, slotid, size;
	local inventory = {};

	for bagid = 0, 4, 1 do
		inventory[bagid] = {};

		size = GetContainerNumSlots(bagid);
		if( size ) then
			for slotid = size, 1, -1 do
				inventory[bagid][slotid] = {};

				local link = GetContainerItemLink(bagid, slotid);
				if( link ) then
					local name = Enchantrix_NameFromLink(link);
					local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bagid, slotid);
					if ((not itemCount) or (itemCount < 1)) then
						itemCount = 1;
					end
					if (name) then
						inventory[bagid][slotid].name = name;
						inventory[bagid][slotid].count = itemCount;
					end
				end
			end
		end
	end
	return inventory;
end

function Enchantrix_DiffInventory(invA, invB)
	local bagid, slotid, size;
	for bagid = 0, 4, 1 do
		size = GetContainerNumSlots(bagid);
		if( size ) then
			for slotid = size, 1, -1 do
				if ((invA[bagid][slotid]) and (invA[bagid][slotid].name)) then
					if (invA[bagid][slotid].name ~= invB[bagid][slotid].name) then
						return invA[bagid][slotid];
					elseif (invA[bagid][slotid].count > invB[bagid][slotid].count) then
						return invA[bagid][slotid];
					end
				end
			end
		end
	end
	return nil;
end

-- Testing script:
-- /script arg1="Disenchant"; arg2="Interface/Icons/INV_Enchant_DustStrange"; Enchantrix_OnEvent("SPELLCAST_START"); Enchantrix_OnEvent("SPELLCAST_STOP"); Enchantrix_DisenchantedItem = {name="Lesser Healing Potion"}; Enchantrix_OnEvent("ITEM_PUSH");
function Enchantrix_OnEvent(event)
	if ((event == "SPELLCAST_START") and (arg1 == "Disenchant")) then
		Enchantrix_Disenchanting = true;
		Enchantrix_WaitingPush = false;
		Enchantrix_StartInv = Enchantrix_TakeInventory();
		Enchantrix_DisenchantCount = 0;
		Enchantrix_DisenchantResult = {};

		return;
	end
	if ((event == "SPELLCAST_FAILED") or (event == "SPELLCAST_INTERRUPTED")) then
		Enchantrix_Disenchanting = false;
		Enchantrix_WaitingPush = false;
		return;
	end
	if ((event == "SPELLCAST_STOP") and (Enchantrix_Disenchanting)) then
		Enchantrix_Disenchanting = false;
		Enchantrix_WaitingPush = true;
		return;
	end
	if ((event == "ITEM_PUSH") and (Enchantrix_WaitingPush)) then
		local textureType = strsub(arg2, 1, 28);
		local receivedItem = strsub(arg2, 29);
		if (not receivedItem) then return; end
		if (textureType == "Interface\\Icons\\INV_Enchant_") then
			local receivedName = ElementNames[receivedItem];
			if (not receivedName) then return; end
			Enchantrix_DisenchantCount = Enchantrix_DisenchantCount + 1;
			Enchantrix_DisenchantResult[Enchantrix_DisenchantCount] = receivedName;
		end
	end
	if ((event == "UNIT_INVENTORY_CHANGED") and (arg1 == "player") and (Enchantrix_DisenchantCount > 0)) then
		local nowInv = Enchantrix_TakeInventory();
		local disenchantedItem = Enchantrix_DiffInventory(Enchantrix_StartInv, nowInv);

		if (not disenchantedItem) then
			return;
		end

		if (not EnchantedLocal[disenchantedItem.name]) then
			EnchantedLocal[disenchantedItem.name] = {};
		end

		local message = "Found that "..disenchantedItem.name.." disenchants into ";
		for i=1, Enchantrix_DisenchantCount, 1 do
			local disenchantResult = Enchantrix_DisenchantResult[i];
			local prevCount = EnchantedLocal[disenchantedItem.name][disenchantResult];
			if (not prevCount) then prevCount = 0; end
			EnchantedLocal[disenchantedItem.name][disenchantResult] = prevCount + 1;
			if (i == 1) then
				message = message..disenchantResult;
			elseif (i == Enchantrix_DisenchantResult) then
				message = message.." and "..disenchantResult;
			else
				message = message..", "..disenchantResult;
			end
		end
		if ( DEFAULT_CHAT_FRAME ) then 
			DEFAULT_CHAT_FRAME:AddMessage(message, 0.8, 0.8, 0.2);
		end
		Enchantrix_DisenchantCount = 0;
		Enchantrix_DisenchantResult = {};
		Enchantrix_Disenchanting = false;
		Enchantrix_WaitingPush = false;
	end
end
